<?xml version="1.0" encoding="iso-8859-1"?>
<!--
  Author: Lloyd McKenzie, IBM
  Date: August 24, 2001

  Description:
    Sample 'FO' stylesheet for DynamicDef profiles.  Allows
    DynamicDef profiles to be turned into PDF or other document
    formats.

    Uses the 'params-fo.xsl' file to set certain options.

    This stylesheet is a first-draft, proof-of concept.  There are
    likely better and more elegant ways to accomplish some of the
    formatting performed.  Appologies in advance for the lack of
    documentation

    If you feel the urge to modify/improve upon this stylesheet,
    I'd appreciate a copy of your work so that I can continue to
    enhance this version. (e-mail: lmckenzi@ca.ibm.com)

  Notes:
    This stylesheet does not display FieldID or ItemNo 
    (One is the HL7 ID.  Don't know what the other is . . .)
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
		xmlns:fo="http://www.w3.org/1999/XSL/Format" 
         	xmlns:fox="http://xml.apache.org/fop/extensions"
		xmlns:java="http://xml.apache.org/xslt/java">

	<xsl:include href="params-fo.xsl"/>
	<xsl:include href="DocInfo.xsl"/>
	<xsl:template match="/">
		<fo:root xmlns:fo="http://www.w3.org/1999/XSL/Format">
			<!-- defines the layout master -->
			<fo:layout-master-set>
				<fo:simple-page-master master-name="first" page-height="11in" page-width="8.5in" margin-top=".4in" margin-bottom=".4in" margin-left=".75in" margin-right=".75in">
					<fo:region-body margin-top=".6in" margin-bottom="0.4in"/>
					<fo:region-before extent=".75in"/>
					<fo:region-after extent="0.4in"/>
				</fo:simple-page-master>
			</fo:layout-master-set>
			
			<!-- beginning of the PDF outline extensions --> 

			<!-- starts actual layout -->
			<fo:page-sequence master-reference="first" initial-page-number="1">

				<!-- header -->
				<fo:static-content flow-name="xsl-region-before">
					<fo:table border-style="none">
						<fo:table-column column-width="1in"/>
						<fo:table-column column-width="6in"/>
						<fo:table-body>
							<fo:table-row>
								<fo:table-cell>
									<fo:block>
										<fo:external-graphic height=".6in">
											<xsl:attribute name="src">
												<xsl:value-of select="$HeaderGraphic"/>
											</xsl:attribute>
										</fo:external-graphic>
									</fo:block>
								</fo:table-cell>
								<fo:table-cell>
									<fo:block text-align="end" font-size="14pt" font-family="serif" line-height="14pt">
										<xsl:value-of select="HL7v2xConformanceProfile/MetaData/@OrgName"/>
									</fo:block>
									<!---
                  <fo:block text-align="end" 
                            font-size="14pt" 
                            font-family="serif" 
                            line-height="14pt"
                            font-style="italic">
                    <xsl:value-of select="$DocName"/>
                  </fo:block>
                  -->
									<fo:block text-align="end" font-size="14pt" font-family="serif" line-height="14pt" font-style="italic">
										<xsl:value-of select="HL7v2xConformanceProfile/MetaData/@Name"/>
									</fo:block>
								</fo:table-cell>
							</fo:table-row>
						</fo:table-body>
					</fo:table>
					<fo:block>
						<fo:leader leader-pattern="rule" rule-thickness="1.0pt" leader-length="7in"/>
					</fo:block>
				</fo:static-content>
				<!-- footer -->
				<fo:static-content flow-name="xsl-region-after">
					<fo:block>
						<fo:leader leader-pattern="rule" rule-thickness="1.0pt" leader-length="7in"/>
					</fo:block>
					<fo:table border-style="none" font-family="serif" font-size="10pt">
						<fo:table-column column-width="2in"/>
						<fo:table-column column-width="3in"/>
						<fo:table-column column-width="2in"/>
						<fo:table-body>
							<fo:table-row>
								<fo:table-cell>
									<fo:block font-family="serif" font-size="10pt">
										<xsl:variable name="date" select="java:java.util.Date.new()"/>
										<xsl:value-of select="java:toLocaleString($date)"/>
									</fo:block>
								</fo:table-cell>
								<fo:table-cell>
									<fo:block text-align="center" font-family="serif" font-size="10pt">
										<xsl:text>Version </xsl:text>
										<xsl:value-of select="HL7v2xConformanceProfile/MetaData/@Version"/>
										<xsl:if test="HL7v2xConformanceProfile/MetaData/@Status!=''">
											<xsl:text> - </xsl:text>
											<xsl:value-of select="HL7v2xConformanceProfile/MetaData/@Status"/>
										</xsl:if>
									</fo:block>
								</fo:table-cell>
								<fo:table-cell>
									<fo:block text-align="end" font-family="serif" font-size="10pt">
                    Page: <fo:page-number/>
									</fo:block>
								</fo:table-cell>
							</fo:table-row>
						</fo:table-body>
					</fo:table>
				</fo:static-content>
				<!-- 'main' text -->
				<fo:flow flow-name="xsl-region-body">
					<xsl:apply-templates/>
				</fo:flow>
			</fo:page-sequence>
		</fo:root>
	</xsl:template>
	<xsl:template match="HL7v2xConformanceProfile">
		<xsl:apply-templates select="." mode="DocInfo"/>
		<fo:block font-size="14pt" font-family="sans-serif" font-weight="bold" keep-with-next="always">
			<xsl:attribute name="id">
				<xsl:text>MessageProfile</xsl:text>
			</xsl:attribute>		
    		</fo:block>
		<fox:outline internal-destination="MessageProfile">
    				<fox:label>Message Profile</fox:label>
  		</fox:outline>
		<xsl:if test="string-length(normalize-space(ImpNote))>1 and $DisplayNotes='true'">
			<fo:block start-indent=".5in" font-family="serif" font-size="10pt" space-after.optimum="6pt">
				<xsl:value-of select="ImpNote"/>
			</fo:block>
		</xsl:if>
		<fo:block font-size="14pt" font-family="sans-serif" font-weight="bold" keep-with-next="always">
      Parameters
    </fo:block>
		<fo:block  start-indent=".25in" keep-with-previous="always" keep-with-next="always" font-style="italic">
     Message Profile
    </fo:block>
		<xsl:call-template name="ShowProperty">
			<xsl:with-param name="Label">HL7 Version</xsl:with-param>
			<xsl:with-param name="Value">
				<xsl:value-of select="@HL7Version"/>
			</xsl:with-param>
		</xsl:call-template>
		<xsl:call-template name="ShowProperty">
			<xsl:with-param name="Label">Profile Type</xsl:with-param>
			<xsl:with-param name="Value">
				<xsl:value-of select="@ProfileType"/>
			</xsl:with-param>
		</xsl:call-template>
		<xsl:call-template name="ShowProperty">
			<xsl:with-param name="Label">Role</xsl:with-param>
			<xsl:with-param name="Value">
				<xsl:value-of select="@Role"/>
			</xsl:with-param>
		</xsl:call-template>
		<xsl:call-template name="ShowProperty">
			<xsl:with-param name="Label">Identifier</xsl:with-param>
			<xsl:with-param name="Value">
				<xsl:value-of select="@Identifier"/>
			</xsl:with-param>
		</xsl:call-template>
		<xsl:call-template name="ShowProperty">
			<xsl:with-param name="Label">Topics</xsl:with-param>
			<xsl:with-param name="Value">
				<xsl:value-of select="MetaData/@Topics"/>
			</xsl:with-param>
		</xsl:call-template>
		<fo:block start-indent=".25in" keep-with-next="always" space-before="6pt" font-style="italic">
      Encoding Method
      <xsl:if test="count(Encodings/Encoding)>1">
        (s)
      </xsl:if>
		</fo:block>
		<fo:block start-indent=".5in" font-size="10pt">
			<xsl:value-of select="Encodings/Encoding[1]"/>
			<xsl:for-each select="Encodings/Encoding[position()!=1]">
				<xsl:text>, </xsl:text>
				<xsl:value-of select="."/>
			</xsl:for-each>
		</fo:block>
		<xsl:apply-templates mode="UseCase"/>
		<xsl:for-each select="HL7v2xStaticDef | HL7v2xStaticDefRef">
			<fo:block start-indent=".25in" space-before="6pt" font-style="italic">
				<fo:basic-link color="blue">
					<xsl:attribute name="internal-destination">
						<xsl:value-of select="generate-id(.)"/>
					</xsl:attribute>
					<xsl:text>Interaction </xsl:text>
					<xsl:value-of select="position()"/>
				</fo:basic-link>
    			</fo:block>	
		</xsl:for-each>
 					
		<xsl:for-each select="HL7v2xStaticDef | HL7v2xStaticDefRef">

			<fo:block font-size="14pt" font-family="sans-serif" font-weight="bold" break-before="page" space-before.optimum="12pt" keep-with-next="always">
				<xsl:attribute name="id">
					<xsl:value-of select="generate-id(.)"/>
				</xsl:attribute>
				<xsl:text>Interaction </xsl:text>
				<xsl:value-of select="position()"/>
    			</fo:block>
  				<fox:outline>
  					<xsl:attribute name="internal-destination">
						<xsl:value-of select="generate-id(.)"/>
  					</xsl:attribute>
   				 	<fox:label>
						<xsl:text>Interaction </xsl:text>
						<xsl:value-of select="position()"/>
   				 	</fox:label>
  				</fox:outline>				

			<xsl:apply-templates select="preceding-sibling::DynamicDef[1]"/>						<xsl:apply-templates select="."/>
		</xsl:for-each>
	</xsl:template>
      <xsl:template match="DynamicDef">
		<fo:block start-indent=".25in" keep-with-next="always" space-before="6pt" font-style="italic">
				<xsl:attribute name="id">
					<xsl:value-of select="generate-id(.)"/>
				</xsl:attribute>
				<xsl:text>Dynamic Definition</xsl:text>  				
    		</fo:block>
		<xsl:call-template name="ShowProperty">
			<xsl:with-param name="Label">Dynamic Profile ID</xsl:with-param>
			<xsl:with-param name="Value">
				<xsl:value-of select="./@DynamicID"/>
			</xsl:with-param>
		</xsl:call-template>
		<xsl:call-template name="ShowProperty">
			<xsl:with-param name="Label">Accept Acknowledgement</xsl:with-param>
			<xsl:with-param name="Value">
				<xsl:value-of select="./@AccAck"/>
			</xsl:with-param>
		</xsl:call-template>
		<xsl:call-template name="ShowProperty">
			<xsl:with-param name="Label">Application Acknowledgement</xsl:with-param>
			<xsl:with-param name="Value">
				<xsl:value-of select="./@AppAck"/>
			</xsl:with-param>
		</xsl:call-template>
		<xsl:call-template name="ShowProperty">
			<xsl:with-param name="Label">Acknowledgement Mode</xsl:with-param>
			<xsl:with-param name="Value">
				<xsl:value-of select="./@MsgAckMode"/>
			</xsl:with-param>
		</xsl:call-template>
	</xsl:template>
      <xsl:template match="HL7v2xStaticDefRef">
		<fo:block start-indent=".25in" keep-with-previous="always" keep-with-next="always" space-before="6pt" font-style="italic">
     Static Definition Reference
    		</fo:block>
		<xsl:call-template name="ShowProperty">
			<xsl:with-param name="Label">Identifier</xsl:with-param>
			<xsl:with-param name="Value">
				<xsl:value-of select="@Identifier"/>
			</xsl:with-param>
		</xsl:call-template>
	</xsl:template>
      <xsl:template match="HL7v2xStaticDef">
		<fo:block start-indent=".25in" keep-with-previous="always" keep-with-next="always" space-before="6pt" font-style="italic">
     Static Definition
    		</fo:block>
		<xsl:call-template name="ShowProperty">
			<xsl:with-param name="Label">Event Description</xsl:with-param>
			<xsl:with-param name="Value">
				<xsl:value-of select="@EventDesc"/>
			</xsl:with-param>
		</xsl:call-template>		
		<xsl:call-template name="ShowProperty">
			<xsl:with-param name="Label">ID</xsl:with-param>
			<xsl:with-param name="Value">
				<xsl:value-of select="@StaticID"/>
			</xsl:with-param>
		</xsl:call-template>
		<xsl:call-template name="ShowProperty">
			<xsl:with-param name="Label">Message Type</xsl:with-param>
			<xsl:with-param name="Value">
				<xsl:value-of select="@MsgType"/>
			</xsl:with-param>
		</xsl:call-template>
		<xsl:call-template name="ShowProperty">
			<xsl:with-param name="Label">Trigger Event</xsl:with-param>
			<xsl:with-param name="Value">
				<xsl:value-of select="@EventType"/>
			</xsl:with-param>
		</xsl:call-template>
		<xsl:call-template name="ShowProperty">
			<xsl:with-param name="Label">Message Structure</xsl:with-param>
			<xsl:with-param name="Value">
				<xsl:value-of select="@MsgStructID"/>
			</xsl:with-param>
		</xsl:call-template>
		<xsl:call-template name="ShowProperty">
			<xsl:with-param name="Label">Order Control Code</xsl:with-param>
			<xsl:with-param name="Value">
				<xsl:value-of select="@OrderControl"/>
			</xsl:with-param>
		</xsl:call-template>
		<xsl:call-template name="ShowProperty">
			<xsl:with-param name="Label">Identifier</xsl:with-param>
			<xsl:with-param name="Value">
				<xsl:value-of select="@Identifier"/>
			</xsl:with-param>
		</xsl:call-template>
		<xsl:call-template name="ShowProperty">
			<xsl:with-param name="Label">Topics</xsl:with-param>
			<xsl:with-param name="Value">
				<xsl:value-of select="./MetaData/@Topics"/>
			</xsl:with-param>
		</xsl:call-template>
		<fo:block start-indent=".25in" keep-with-previous="always" font-style="italic">
      Grammar
    		</fo:block>
		<fo:block start-indent=".5in" font-size="10pt" space-after.optimum="6pt">
			<xsl:apply-templates select="." mode="HL7Grammar"/>
		</fo:block>
		<xsl:apply-templates/>
	</xsl:template>

	<xsl:template match="SegGroup" mode="HL7Grammar">
		<fo:basic-link color="blue">
			<xsl:attribute name="internal-destination">
				<xsl:value-of select="generate-id(.)"/>
			</xsl:attribute>
			<xsl:if test="@Usage!='R'">
				<xsl:text>[</xsl:text>
			</xsl:if>
			<xsl:if test="@Max!='1'">
				<xsl:text>{</xsl:text>
			</xsl:if>
		</fo:basic-link>
		<xsl:apply-templates mode="HL7Grammar"/>
		<fo:basic-link color="blue">
			<xsl:attribute name="internal-destination">
				<xsl:value-of select="generate-id(.)"/>-END
      			</xsl:attribute>
			<xsl:if test="@Max!='1'">
				<xsl:text>}</xsl:text>
			</xsl:if>
			<xsl:if test="@Usage!='R'">
				<xsl:text>]</xsl:text>
			</xsl:if>
		</fo:basic-link>
		<xsl:text/>
	</xsl:template>
	<xsl:template match="Segment" mode="HL7Grammar">
		<xsl:if test="@Max!='1'">
			<xsl:text>{</xsl:text>
		</xsl:if>
		<xsl:if test="@Usage!='R'">
			<xsl:text>[</xsl:text>
		</xsl:if>
		<fo:basic-link color="blue">
			<xsl:attribute name="internal-destination">
				<xsl:value-of select="generate-id(.)"/>
			</xsl:attribute>
			<xsl:value-of select="@Name"/>
		</fo:basic-link>
		<xsl:if test="@Usage!='R'">
			<xsl:text>]</xsl:text>
		</xsl:if>
		<xsl:if test="@Max!='1'">
			<xsl:text>}</xsl:text>
		</xsl:if>
		<xsl:text/>
  				<fox:outline>
  					<xsl:attribute name="internal-destination">
						<xsl:value-of select="generate-id(.)"/>
  					</xsl:attribute>
   				 	<fox:label>
						<xsl:value-of select="@Name"/>
   				 	</fox:label>
  				</fox:outline>				

	</xsl:template>
	<xsl:template match="*[self::Segment or self::SegGroup]">
		<!-- this defines a title level 2-->
		<fo:block keep-with-next="always" font-size="12pt" font-family="sans-serif" font-weight="bold" space-before.optimum="6pt">
			<xsl:attribute name="id">
				<xsl:value-of select="generate-id(.)"/>
			</xsl:attribute>
			<xsl:if test="self::node()[self::SegGroup]">
        &lt;
      </xsl:if>
			<xsl:value-of select="@Name"/>
			<xsl:if test="@LongName!=''">
				<xsl:text> - </xsl:text>
				<xsl:value-of select="@LongName"/>
			</xsl:if>
			<xsl:if test="(string-length(normalize-space(Description))>1)">
				<xsl:text> - </xsl:text>
				<xsl:value-of select="Description"/>
			</xsl:if>
			<xsl:if test="self::node()[self::SegGroup]">
        &gt;
      </xsl:if>
		</fo:block>
		<fo:block start-indent=".25in" keep-with-previous="always" keep-with-next="always" font-style="italic" font-size="10pt">
			<xsl:text>(Usage: </xsl:text>
			<fo:inline font-style="normal">
				<xsl:call-template name="translate-Usage">
					<xsl:with-param name="Usage-code">
						<xsl:value-of select="@Usage"/>
					</xsl:with-param>
				</xsl:call-template>
			</fo:inline>
      &#x00A0;&#x00A0;<xsl:text>Cardinality:</xsl:text>
			<fo:inline font-style="normal">
				<xsl:call-template name="find-cardinality">
					<xsl:with-param name="Usage">
						<xsl:value-of select="@Usage"/>
					</xsl:with-param>
					<xsl:with-param name="min">
						<xsl:value-of select="@Min"/>
					</xsl:with-param>
					<xsl:with-param name="max">
						<xsl:value-of select="@Max"/>
					</xsl:with-param>
				</xsl:call-template>
			</fo:inline>
			<xsl:text>)</xsl:text>
		</fo:block>
		<!-- normal text -->
		<xsl:if test="string-length(normalize-space(ImpNote))>1 and $DisplayNotes='true'">
			<fo:block text-align="start" keep-with-previous="always" space-after.optimum="3pt" start-indent=".5in" font-family="serif" font-size="10pt">
				<xsl:value-of select="ImpNote"/>
			</fo:block>
		</xsl:if>
		<xsl:if test="(string-length(normalize-space(Predicate))>1) and $DisplayPredicate='true'">
			<fo:block start-indent=".25in" keep-with-next="always" font-style="italic">
				<xsl:attribute name="id">
					<xsl:value-of select="generate-id(Predicate)"/>
				</xsl:attribute>
        Condition Predicate:
      </fo:block>
			<fo:block start-indent=".5in" keep-with-previous="always" space-before.optimum="6pt" font-family="serif" font-size="10pt">
				<xsl:value-of select="Predicate"/>
			</fo:block>
		</xsl:if>
		<xsl:choose>
			<xsl:when test="self::node()[self::Segment]">
				<xsl:apply-templates select="." mode="table"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="./*"/>
				<fo:block font-size="12pt" font-family="sans-serif" font-weight="bold" text-align="start" space-before="12pt">
					<xsl:attribute name="id">
						<xsl:value-of select="generate-id(.)"/>-END</xsl:attribute>
					<xsl:text>&lt;End </xsl:text>
					<xsl:value-of select="@Name"/>
					<xsl:if test="(string-length(normalize-space(Description))>1)">
						<xsl:text> - </xsl:text>
						<xsl:value-of select="Description"/>
					</xsl:if>
					<xsl:text>&gt;</xsl:text>
				</fo:block>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<xsl:template match="UseCase" mode="UseCase">
		<fo:block font-size="14pt" font-family="sans-serif" font-weight="bold" space-before.optimum="12pt" keep-with-next="always">
      Use Case
    </fo:block>
		<fo:block start-indent=".5in" font-family="serif" font-size="10pt" space-after.optimum="6pt">
			<xsl:value-of select="UseCase"/>
		</fo:block>
		<fo:block start-indent=".25in" font-style="italic">
      Actors
    </fo:block>
		<fo:block start-indent=".5in" font-size="10pt" space-after.optimum="6pt">
			<fo:table wrap-option="wrap" overflow="hidden">
				<fo:table-column column-width="1.5in"/>
				<fo:table-column column-width="5.5in"/>
				<fo:table-body>
					<xsl:for-each select="./Actor">
						<fo:table-row>
							<fo:table-cell>
								<fo:block space-after="6pt">
									<xsl:value-of select="@Name"/>
									<xsl:text>:</xsl:text>
								</fo:block>
							</fo:table-cell>
							<fo:table-cell>
								<fo:block font-family="serif">
									<xsl:value-of select="."/>
								</fo:block>
							</fo:table-cell>
						</fo:table-row>
					</xsl:for-each>
				</fo:table-body>
			</fo:table>
		</fo:block>
		<fo:block start-indent=".25in" font-style="italic">
      Pre Conditions
    </fo:block>
		<fo:block start-indent=".5in" font-size="10pt" space-after.optimum="6pt">
			<fo:table wrap-option="wrap" overflow="hidden">
				<fo:table-column column-width="1.5in"/>
				<fo:table-column column-width="5.5in"/>
				<fo:table-body>
					<xsl:for-each select="./PreCondition">
						<fo:table-row>
							<fo:table-cell>
								<fo:block space-after="6pt">
									<xsl:value-of select="@Name"/>
									<xsl:text>:</xsl:text>
								</fo:block>
							</fo:table-cell>
							<fo:table-cell>
								<fo:block font-family="serif">
									<xsl:value-of select="."/>
								</fo:block>
							</fo:table-cell>
						</fo:table-row>
					</xsl:for-each>
				</fo:table-body>
			</fo:table>
		</fo:block>
		<fo:block start-indent=".25in" font-style="italic">
      Event Flow
    </fo:block>
		<fo:block start-indent=".5in" font-size="10pt" space-after.optimum="6pt">
			<fo:table wrap-option="wrap" overflow="hidden">
				<fo:table-column column-width="1.5in"/>
				<fo:table-column column-width="5.5in"/>
				<fo:table-body>
					<xsl:for-each select="./EventFlow">
						<fo:table-row>
							<fo:table-cell>
								<fo:block space-after="6pt">
									<xsl:value-of select="@Name"/>
									<xsl:text>:</xsl:text>
								</fo:block>
							</fo:table-cell>
							<fo:table-cell>
								<fo:block font-family="serif">
									<xsl:value-of select="."/>
								</fo:block>
							</fo:table-cell>
						</fo:table-row>
					</xsl:for-each>
				</fo:table-body>
			</fo:table>
		</fo:block>
		<fo:block start-indent=".25in" font-style="italic">
      Post Conditions
    </fo:block>
		<fo:block start-indent=".5in" font-size="10pt" space-after.optimum="6pt">
			<fo:table wrap-option="wrap" overflow="hidden">
				<fo:table-column column-width="1.5in"/>
				<fo:table-column column-width="5.5in"/>
				<fo:table-body>
					<xsl:for-each select="./PostCondition">
						<fo:table-row>
							<fo:table-cell>
								<fo:block space-after="6pt">
									<xsl:value-of select="@Name"/>
									<xsl:text>:</xsl:text>
								</fo:block>
							</fo:table-cell>
							<fo:table-cell>
								<fo:block font-family="serif">
									<xsl:value-of select="."/>
								</fo:block>
							</fo:table-cell>
						</fo:table-row>
					</xsl:for-each>
				</fo:table-body>
			</fo:table>
		</fo:block>
		<fo:block start-indent=".25in" font-style="italic">
      Derived Events
    </fo:block>
		<fo:block start-indent=".5in" font-size="10pt" space-after.optimum="6pt">
			<fo:table wrap-option="wrap" overflow="hidden">
				<fo:table-column column-width="1.5in"/>
				<fo:table-column column-width="5.5in"/>
				<fo:table-body>
					<xsl:for-each select="./DerivedEvent">
						<fo:table-row>
							<fo:table-cell>
								<fo:block space-after="6pt">
									<xsl:value-of select="@Name"/>
									<xsl:text>:</xsl:text>
								</fo:block>
							</fo:table-cell>
							<fo:table-cell>
								<fo:block font-family="serif">
									<xsl:value-of select="."/>
								</fo:block>
							</fo:table-cell>
						</fo:table-row>
					</xsl:for-each>
				</fo:table-body>
			</fo:table>
		</fo:block>
	</xsl:template>
	<xsl:template match="UseCase/*" mode="UseCase">
		<fo:list-item font-family="serif" font-size="10pt">
			<!-- insert a bullet -->
			<fo:list-item-label end-indent="label-end()">
				<fo:block text-align="end">
					<xsl:number level="single" from="UseCase" format="1. "/>
				</fo:block>
			</fo:list-item-label>
			<!-- list text -->
			<fo:list-item-body start-indent="body-start()">
				<fo:block>
					<xsl:text>Where Am I</xsl:text>
					<xsl:value-of select="UseCase"/>
				</fo:block>
			</fo:list-item-body>
		</fo:list-item>
	</xsl:template>
	<xsl:template match="Segment" mode="table">
		<!-- table start -->
		<fo:block keep-with-previous="always">
			<fo:table border-style="solid" border-width="2pt" wrap-option="wrap" overflow="hidden" margin="2" space-before.optimum="6pt" space-after.optimum="6pt" font-size="10pt">
				<fo:table-column padding="2pt" column-width=".45in"/>
				<!-- Sequence -->
				<fo:table-column column-width="2.5in"/>
				<!-- Name -->
				<fo:table-column column-width=".6in"/>
				<!-- Type -->
				<fo:table-column column-width=".6in"/>
				<!-- Table -->
				<fo:table-column column-width=".4in"/>
				<!-- Length -->
				<fo:table-column column-width=".35in"/>
				<!-- Usage -->
				<fo:table-column column-width=".4in"/>
				<!-- Cardinality -->
				<xsl:if test="$DisplayValues='true'">
					<fo:table-column column-width="1.7in"/>
					<!-- Contents -->
				</xsl:if>
				<fo:table-header>
					<fo:table-row border-style="solid" border-width="2pt">
						<fo:table-cell border-style="solid" border-width=".5pt">
							<fo:block font-weight="bold" text-align="center" vertical-align="middle" border-width="2pt" border-color="black" background-color="#E0E0E0">
              Seq.
              </fo:block>
						</fo:table-cell>
						<fo:table-cell border-style="solid" border-width=".5pt">
							<fo:block font-weight="bold" text-align="center" vertical-align="middle" border-width="2pt" border-color="black" background-color="#E0E0E0">
              Name
              </fo:block>
						</fo:table-cell>
						<fo:table-cell border-style="solid" border-width=".5pt">
							<fo:block font-weight="bold" text-align="center" vertical-align="middle" border-width="2pt" border-color="black" background-color="#E0E0E0">
              Type
              </fo:block>
						</fo:table-cell>
						<fo:table-cell border-style="solid" border-width=".5pt">
							<fo:block font-weight="bold" text-align="center" vertical-align="middle" border-width="2pt" border-color="black" background-color="#E0E0E0">
              Table
              </fo:block>
						</fo:table-cell>
						<fo:table-cell border-style="solid" border-width=".5pt">
							<fo:block font-weight="bold" text-align="center" vertical-align="middle" border-width="2pt" border-color="black" background-color="#E0E0E0">
              Len.
              </fo:block>
						</fo:table-cell>
						<fo:table-cell border-style="solid" border-width=".5pt">
							<fo:block font-weight="bold" text-align="center" vertical-align="middle" border-width="2pt" border-color="black" background-color="#E0E0E0">
              Opt.
              </fo:block>
						</fo:table-cell>
						<fo:table-cell border-style="solid" border-width=".5pt">
							<fo:block font-weight="bold" text-align="center" vertical-align="middle" border-width="2pt" border-color="black" background-color="#E0E0E0">
              Card.
              </fo:block>
						</fo:table-cell>
						<xsl:if test="$DisplayValues='true'">
							<fo:table-cell border-style="solid" border-width=".5pt">
								<fo:block font-weight="bold" text-align="center" vertical-align="middle" border-width="2pt" border-color="black" background-color="#E0E0E0">
              Contents
              </fo:block>
							</fo:table-cell>
						</xsl:if>
					</fo:table-row>
				</fo:table-header>
				<fo:table-body>
					<xsl:apply-templates mode="table"/>
				</fo:table-body>
			</fo:table>
		</fo:block>
		<!-- table end -->
		<xsl:apply-templates mode="list"/>
	</xsl:template>
	<xsl:template match="ImpNote" mode="table"/>
	<xsl:template mode="table" match="*[self::Field or self::Component or self::SubComponent]">
		<xsl:if test="($IncludeNotSupported='true' or (@Usage!='X' and @Usage!='NS')) and ($FieldsOnly='false' or self::Field)">
			<fo:table-row>
				<xsl:if test="ancestor-or-self::*[@Usage='X'] or ancestor-or-self::*[@Usage='NS']">
					<xsl:attribute name="background-color">
						<xsl:text>#E0E0E0</xsl:text>
					</xsl:attribute>
				</xsl:if>
				<fo:table-cell padding="2pt" border-style="solid" border-width=".5pt">
					<fo:block font-family="serif">
						<xsl:choose>
							<xsl:when test="(string-length(normalize-space(ImpNote))>1 and $DisplayNotes='true') or (string-length(normalize-space(Description))>1) or ((string-length(normalize-space(Predicate))>1) and $DisplayPredicate='true') or ((string-length(normalize-space(Reference))>1) and $DisplayReference='true')">
								<fo:basic-link color="blue">
									<xsl:attribute name="internal-destination">
										<xsl:value-of select="generate-id(.)"/>
									</xsl:attribute>
									<xsl:number level="multiple" count="*[self::Field or self::Component or self::SubComponent]"/>
								</fo:basic-link>
							</xsl:when>
							<xsl:otherwise>
								<xsl:number level="multiple" count="*[self::Field or self::Component or self::SubComponent]"/>
							</xsl:otherwise>
						</xsl:choose>
					</fo:block>
				</fo:table-cell>
				<fo:table-cell padding="2pt" border-style="solid" border-width=".5pt">
					<fo:block font-family="serif">
						<xsl:attribute name="start-indent">
							<xsl:choose>
								<xsl:when test="self::Component">
                  .1in
                </xsl:when>
								<xsl:when test="self::SubComponent">
                  .2in
                </xsl:when>
								<xsl:otherwise>
                  0in
                </xsl:otherwise>
							</xsl:choose>
						</xsl:attribute>
						<xsl:value-of select="@Name"/>
					</fo:block>
				</fo:table-cell>
				<fo:table-cell padding="2pt" border-style="solid" border-width=".5pt">
					<fo:block font-family="serif">
						<xsl:value-of select="@Datatype"/>
					</fo:block>
				</fo:table-cell>
				<fo:table-cell padding="2pt" border-style="solid" border-width=".5pt">
					<fo:block text-align="center" font-family="serif">
						<xsl:if test="@Table">
							<xsl:if test="(substring(@Table,1,1))=0">
								<xsl:text>HL7</xsl:text>
							</xsl:if>
						</xsl:if>
						<xsl:value-of select="@Table"/>
					</fo:block>
				</fo:table-cell>
				<fo:table-cell padding="2pt" border-style="solid" border-width=".5pt">
					<fo:block text-align="end" font-family="serif">
						<xsl:choose>
							<xsl:when test="@Usage!='X'">
								<xsl:value-of select="@Length"/>
							</xsl:when>
							<xsl:otherwise/>
						</xsl:choose>
					</fo:block>
				</fo:table-cell>
				<fo:table-cell padding="2pt" border-style="solid" border-width=".5pt">
					<fo:block text-align="center" font-family="serif">
						<xsl:choose>
							<xsl:when test="(string-length(normalize-space(Predicate))>1) and $DisplayPredicate='true'">
								<fo:basic-link color="blue">
									<xsl:attribute name="internal-destination">
										<xsl:value-of select="generate-id(Predicate)"/>
									</xsl:attribute>
									<xsl:value-of select="@Usage"/>
								</fo:basic-link>
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of select="@Usage"/>
							</xsl:otherwise>
						</xsl:choose>
					</fo:block>
				</fo:table-cell>
				<fo:table-cell padding="2pt" border-style="solid" border-width=".5pt">
					<fo:block text-align="center" font-family="serif">
						<xsl:call-template name="find-cardinality">
							<xsl:with-param name="Usage">
								<xsl:value-of select="@Usage"/>
							</xsl:with-param>
							<xsl:with-param name="min">
								<xsl:value-of select="@Min"/>
							</xsl:with-param>
							<xsl:with-param name="max">
								<xsl:value-of select="@Max"/>
							</xsl:with-param>
						</xsl:call-template>
					</fo:block>
				</fo:table-cell>
				<xsl:if test="$DisplayValues='true'">
					<fo:table-cell padding="2pt" border-style="solid" border-width=".5pt">
						<xsl:choose>
							<xsl:when test="./DataValues">
								<xsl:for-each select="./DataValues">
									<fo:block text-align="start" font-family="serif">
										<xsl:if test="@ExValue!=''">
											<xsl:choose>
												<xsl:when test="@Fixed='True'">
													<xsl:attribute name="font-weight">bold</xsl:attribute>
												</xsl:when>
												<xsl:otherwise>
													<xsl:text>e.g. </xsl:text>
												</xsl:otherwise>
											</xsl:choose>
											<fo:inline font-family="Courier">
												<xsl:value-of select="@ExValue"/>
											</fo:inline>
										</xsl:if>
									</fo:block>
								</xsl:for-each>
							</xsl:when>
							<xsl:otherwise>
								<fo:block/>
							</xsl:otherwise>
						</xsl:choose>
					</fo:table-cell>
				</xsl:if>
			</fo:table-row>
			<xsl:apply-templates mode="table"/>
		</xsl:if>
	</xsl:template>
	<xsl:template mode="list" match="*[self::Field or self::Component or self::SubComponent]">
		<xsl:if test="($IncludeNotSupported='true' or (@Usage!='X' and @Usage!='NS')) and ($FieldsOnly='false' or self::Field)">
			<xsl:if test="(string-length(normalize-space(ImpNote))>1 and $DisplayNotes='true') or (string-length(normalize-space(Description))>1)or ((string-length(normalize-space(Predicate))>1) and $DisplayPredicate='true') or ((string-length(normalize-space(Reference))>1) and $DisplayReference='true')">
				<fo:block>
					<xsl:if test="ancestor-or-self::*[@Usage='X'] or ancestor-or-self::*[@Usage='NS']">
						<xsl:attribute name="background-color">
							<xsl:text>#E0E0E0</xsl:text>
						</xsl:attribute>
					</xsl:if>
					<fo:block keep-with-next="always" font-size="11pt">
						<xsl:attribute name="id">
							<xsl:value-of select="generate-id(.)"/>
						</xsl:attribute>
						<xsl:number level="multiple" count="*[self::Field or self::Component or self::SubComponent]"/>
						<xsl:text>. </xsl:text>
						<xsl:value-of select="@Name"/>
					</fo:block>
					<xsl:if test="string-length(normalize-space(Description))>1">
						<fo:block start-indent=".5in" keep-with-previous="always" space-after.optimum="6pt" font-family="serif" font-size="10pt">
							<xsl:value-of select="normalize-space(Description)"/>
						</fo:block>
					</xsl:if>
					<xsl:if test="string-length(normalize-space(ImpNote))>1 and $DisplayNotes='true'">
						<fo:block start-indent=".5in" keep-with-previous="always" space-after.optimum="6pt" font-family="serif" font-size="10pt">
							<xsl:value-of select="normalize-space(ImpNote)"/>
						</fo:block>
					</xsl:if>
					<xsl:if test="(string-length(normalize-space(Predicate))>1)!='' and $DisplayPredicate='true'">
						<fo:block start-indent=".25in" keep-with-next="always" font-style="italic" font-size="11pt">
							<xsl:attribute name="id">
								<xsl:value-of select="generate-id(Predicate)"/>
							</xsl:attribute>
              Condition Predicate:
            </fo:block>
						<fo:block start-indent=".5in" keep-with-previous="always" space-after.optimum="6pt" font-family="serif" font-size="10pt">
							<xsl:value-of select="Predicate"/>
						</fo:block>
					</xsl:if>
					<xsl:if test="Reference!='' and $DisplayReference='true'">
						<fo:block start-indent=".25in" keep-with-next="always" font-style="italic" font-size="11pt">
              Reference(s):
            </fo:block>
						<fo:block start-indent=".5in" keep-with-previous="always" space-after.optimum="6pt" font-family="serif" font-size="10pt">
							<xsl:value-of select="Reference"/>
						</fo:block>
					</xsl:if>
				</fo:block>
			</xsl:if>
			<xsl:apply-templates mode="list"/>
		</xsl:if>
	</xsl:template>
	<xsl:template name="translate-Usage">
		<xsl:param name="Usage-code"/>
		<xsl:choose>
			<xsl:when test="$Usage-code='R'">Required</xsl:when>
			<xsl:when test="$Usage-code='RE'">Required but may be empty</xsl:when>
			<xsl:when test="$Usage-code='O'">Optional</xsl:when>
			<xsl:when test="$Usage-code='C'">Conditional</xsl:when>
			<xsl:when test="$Usage-code='CE'">Conditionally required but may be empty</xsl:when>
		</xsl:choose>
	</xsl:template>
	<xsl:template name="find-cardinality">
		<xsl:param name="Usage"/>
		<xsl:param name="min"/>
		<xsl:param name="max"/>
		<xsl:choose>
			<xsl:when test="../@Usage='X'"/>
			<xsl:when test="$Usage='X'"/>
			<xsl:when test="$Usage='R' and $max=''">
        1..1
        </xsl:when>
			<xsl:when test="($Usage='RE' or $Usage='O' or $Usage='C') and $max='1'">
        0..1
        </xsl:when>
			<xsl:when test="($Usage='RE' or $Usage='O' or $Usage='C') and ($max=0)">
          0...*
        </xsl:when>
			<xsl:when test="($Usage='RE' or $Usage='O' or $Usage='C') and ($max!='0' and $max!='1')">
				<xsl:text>0..</xsl:text>
				<xsl:value-of select="$max"/>
			</xsl:when>
			<xsl:when test="($Usage='R') and ($max=0)">
				<xsl:text>1..*</xsl:text>
			</xsl:when>
			<xsl:when test="($Usage='R') and ($max!=0)">
				<xsl:text>1..</xsl:text>
				<xsl:value-of select="$max"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$min"/>
				<xsl:text>..</xsl:text>
				<xsl:value-of select="$max"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<xsl:template name="ShowProperty">
		<xsl:param name="Label"/>
		<xsl:param name="Value"/>
		<xsl:if test="$Value!=''">
			<fo:table wrap-option="wrap" overflow="hidden" column-gap="0pt">
				<fo:table-column column-width="2.3in"/>
				<fo:table-column column-width="4.7in"/>
				<fo:table-body>
					<fo:table-row>
						<fo:table-cell>
							<fo:block start-indent=".5in" font-size="10pt">
								<xsl:value-of select="$Label"/>
								<xsl:text>: </xsl:text>
							</fo:block>
						</fo:table-cell>
						<fo:table-cell>
							<fo:block font-family="serif" font-size="10pt">
								<xsl:value-of select="$Value"/>
							</fo:block>
						</fo:table-cell>
					</fo:table-row>
				</fo:table-body>
			</fo:table>
		</xsl:if>
	</xsl:template>
	<xsl:template name="ShowDocProperty">
		<xsl:param name="Label"/>
		<xsl:param name="Value"/>
		<xsl:if test="$Value!=''">
			<fo:table wrap-option="wrap" overflow="hidden" column-gap="0pt" white-space-collapse="false">
				<fo:table-column column-width="2.3in"/>
				<fo:table-column column-width="4.7in"/>
				<fo:table-body>
					<fo:table-row>
						<fo:table-cell>
							<fo:block  font-size="12pt" space-after.optimum="6pt">
								<xsl:value-of select="$Label"/>
								<xsl:text>: </xsl:text>
							</fo:block>
						</fo:table-cell>
						<fo:table-cell>
							<fo:block font-family="serif" font-size="12pt" space-after.optimum="6pt">
								<xsl:value-of select="$Value"/>
							</fo:block>
						</fo:table-cell>
					</fo:table-row>
				</fo:table-body>
			</fo:table>
		</xsl:if>
	</xsl:template>
</xsl:stylesheet>
